/******************************************************************************************************
Copyright (C) 2012 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*****************************************************************************************************/

/*globals BaseConverter, NMRSpectrum, NMRSpectrumUtils, File, TextStream, Env, mainWindow, Dir, print*/
/*jslint plusplus: true, indent: 4*/

/** Default constructor of the converter
 * Defines the file filter
*/
function ASCIIMatrixConverter() {
	"use strict";
	BaseConverter.call(this);
	this.exportFileFilter = "NMR CSV Matrix (Transposed) (*.csv *.txt)"; //!< It will override any other script converter with the same exportFileFilter
}

ASCIIMatrixConverter.prototype = new BaseConverter();

/** Can export a 1D Spectra collection
 */
ASCIIMatrixConverter.prototype.wantsExport = function (aPageItems) {
	"use strict";
	var i, nmrSpc, nmrUtils,
		specs = [];

	for (i = 0; i < aPageItems.length; i++) {
		nmrSpc = new NMRSpectrum(aPageItems[i]);
		if (!nmrSpc.isValid() || nmrSpc.dimCount !== 1) {
			return false;
		}
		specs.push(nmrSpc);
	}
	nmrUtils = new NMRSpectrumUtils();
	return nmrUtils.overlayCompatibility(specs);
};

/** Writes the file
Exports a 1D Spectra collection as an ASCII matrix. Returns true if the file aFilename is generated correctly.
 */
ASCIIMatrixConverter.prototype.exportItems = function (aPageItems, aFilename) {
	"use strict";
	var i, s, nmrSpc, nmrSpec, spectra, nmrUtils, file, strm, dim, scaleBounds, min, max, v, fullBounds, fullMin, fullMax, step,
		distance, n, pos, initPos, posPPM, posPt,
		specs = [],
		isArrayed = false,
		activeSpec = 0;

	for (i = 0; i < aPageItems.length; i++) {
		nmrSpc = new NMRSpectrum(aPageItems[i]);
		if (nmrSpc.isValid() && nmrSpc.dimCount === 1) {
			if (nmrSpc.specCount > 1) { //isArrayed
				isArrayed = true;
				activeSpec = nmrSpc.curSpecIndex;
				spectra = nmrSpc.spectra;
				for (s = 0; s < spectra.length; s++) {
					specs.push(spectra[s]);
				}
			} else {
				specs.push(nmrSpc);
			}
		}
	}
	if (!specs.length) {
		return 0;
	}

	nmrUtils = new NMRSpectrumUtils();
	file = new File(aFilename);
	if (file.open(File.WriteOnly)) {
		strm = new TextStream(file);
		dim = 1;

		scaleBounds = nmrUtils.scaleBoundaries(dim, specs);
		min = scaleBounds.min;
		max = scaleBounds.max;
		if (min > max) {
			v = max;
			max = min;
			min = v;
		}

		fullBounds = nmrUtils.fullScaleBoundaries(dim, specs);
		fullMin = fullBounds.min;
		fullMax = fullBounds.max;
		if (fullMin > fullMax) {
			v = fullMax;
			fullMax = fullMin;
			fullMin = v;
		}
		step = (fullMax - fullMin) / nmrUtils.getMaxNumberPoints(dim, specs);

		distance = Math.abs(min - fullMin);
		n = Math.floor(distance / step);
		pos = fullMin + n * step;
		while (pos < min) {
			pos += step;
		}

		initPos = pos;
		for (s = 0; s < specs.length; s++) {
			nmrSpec = specs[s];
			//UnitsToUnits uses the active spectrum always. Fixed bug #5019.
			nmrSpec.curSpecIndex = s;
			if (s === 0) {
				strm.write("#", "\t", "Title", "\t", "Class", "\t");
				//condition pos-max<step/4: sometimes, because of calculations, the last point is not exported.
				while (pos <= max || Math.abs(pos - max) < step / 4) {
					posPPM = pos;
					if (!nmrSpec.isInHiddenHorzRanges(posPPM)) {
						strm.write(posPPM, "\t");
					}
					pos += step;
				}
				strm.write("\n");
			}
			strm.write(s + 1, "\t", nmrSpec.title, "\t", nmrSpec.className, "\t");
			pos = initPos;
			//condition pos-max<step/4: sometimes, because of calculations, the last point is not exported.
			while (pos <= max || Math.abs(pos - max) < step / 4) {
				//pos is in PPM units
				if (!nmrSpec.isInHiddenHorzRanges(pos)) {
					posPt = nmrSpec.unitsToUnits("ppm", "pt", pos, 1);
					strm.write(nmrSpec.real(posPt), "\t");
				}
				pos += step;
			}
			strm.write("\n");
		}

		if (isArrayed) { //recover the old active spectrum index
			nmrSpec.curSpecIndex = activeSpec;
		}

		file.close();
		return 0;
	}

	return 1;
};

/** Add to the Env.Converters environment engine
 */
Env.Converters.push(new ASCIIMatrixConverter());

//test
ASCIIMatrixConverter.ASCIIMatrixConverterTest = function () {
	"use strict";

	var retVal = -1,
		converter = new ASCIIMatrixConverter(),
		pageItems = mainWindow.activeDocument.selection(),
		file = Dir.home() + "/ASCIIMatrix.txt";

	if (converter.wantsExport(pageItems)) {
		retVal = converter.exportItems(pageItems, file);
	}
	print(retVal + " (" + file + ")");
	return retVal;
};

Env.NMRBinningExport = function (aItem, aFilename) {
	'use strict';
	var converter = new ASCIIMatrixConverter();
	if (converter.wantsExport([aItem])) {
		return converter.exportItems([aItem], aFilename);
	}
	return false;
};
