/******************************************************************************************************
Copyright (C) 2012 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*****************************************************************************************************/

/*globals BaseConverter, NMRSpectrum, NMRSpectrumUtils, File, TextStream, Env, mainWindow, Dir, print*/
/*jslint plusplus: true, indent: 4*/

/** Default constructor of the converter
 * Defines the file filter
*/
function ASCIIMatrixColumnsConverter() {
	"use strict";

	BaseConverter.call(this);
	this.exportFileFilter = "NMR CSV Matrix (*.csv *.txt)"; //!< It will override any other script converter with the same exportFileFilter
}

ASCIIMatrixColumnsConverter.prototype = new BaseConverter();

/** Can export a 1D Spectra collection
 */
ASCIIMatrixColumnsConverter.prototype.wantsExport = function (aPageItems) {
	"use strict";
	var i, nmrSpc, nmrUtils,
		specs = [];

	for (i = 0; i < aPageItems.length; i++) {
		nmrSpc = new NMRSpectrum(aPageItems[i]);
		if (!nmrSpc.isValid() || nmrSpc.dimCount !== 1) {
			return false;
		}
		specs.push(nmrSpc);
	}
	nmrUtils = new NMRSpectrumUtils();
	return nmrUtils.overlayCompatibility(specs);
};

/** Writes the file
Exports a 1D Spectra collection as an ASCII matrix, each spectrum in a column. Returns true if the file aFilename is generated correctly.
 */
ASCIIMatrixColumnsConverter.prototype.exportItems = function (aPageItems, aFilename) {
	"use strict";
	var i, s, v, nmrSpc, nmrSpec, spectra, nmrUtils, file, strm, dim, scaleBounds, min, max, fullBounds, fullMin, fullMax,
		step, distance, n, pos, titles, posPPM, horzUnits, posPt, height,
		specs = [],
		isArrayed = false,
		activeSpec = 0;

	for (i = 0; i < aPageItems.length; i++) {
		nmrSpc = new NMRSpectrum(aPageItems[i]);
		if (nmrSpc.isValid() && nmrSpc.dimCount === 1) {
			if (nmrSpc.specCount > 1) { //isArrayed
				isArrayed = true;
				activeSpec = nmrSpc.curSpecIndex;
				spectra = nmrSpc.spectra;
				for (s = 0; s < spectra.length; s++) {
					specs.push(spectra[s]);
				}
			} else {
				specs.push(nmrSpc);
			}
		}
	}
	if (!specs.length) {
		return 0;
	}

	nmrUtils = new NMRSpectrumUtils();
	file = new File(aFilename);
	if (file.open(File.WriteOnly)) {
		strm = new TextStream(file);
		dim = 1;

		scaleBounds = nmrUtils.scaleBoundaries(dim, specs);
		min = scaleBounds.min;
		max = scaleBounds.max;
		if (min > max) {
			v = max;
			max = min;
			min = v;
		}

		fullBounds = nmrUtils.fullScaleBoundaries(dim, specs);
		fullMin = fullBounds.min;
		fullMax = fullBounds.max;
		if (fullMin > fullMax) {
			v = fullMax;
			fullMax = fullMin;
			fullMin = v;
		}
		step = (fullMax - fullMin) / nmrUtils.getMaxNumberPoints(dim, specs);

		distance = Math.abs(min - fullMin);
		n = Math.floor(distance / step);
		pos = fullMin + n * step;
		while (pos < min) {
			pos += step;
		}

		titles = "";
		if (specs.length > 1) {
			for (s = 0; s < specs.length; s++) {
				nmrSpec = specs[s];
				titles += nmrSpec.title;
				titles += "\t";
				if (s === specs.legth - 1) {
					titles -= "\t";
				}
			}
		}

		//condition pos-max<step/4: sometimes, because of calculations, the last point is not exported.
		if (specs.length > 1) {
			strm.write("\t", titles, "\n");
		}
		while (pos <= max ||  Math.abs(pos - max) < step / 4) {
			for (s = 0; s < specs.length; s++) {
				nmrSpec = specs[s];

				//UnitsToUnits uses the active spectrum always
				nmrSpec.curSpecIndex = s;
				posPPM = pos;
				if (s === 0) {
					strm.write(posPPM, "\t");
				}
				horzUnits = nmrSpec.getProperty("axes.horizontal.units");
				posPt = nmrSpec.unitsToUnits("ppm", "pt", posPPM, 1);
				height = nmrSpec.real(posPt);
				if (nmrSpec.isInHiddenHorzRanges(posPPM)) {
					height = 0.0;
				}
				if (s < specs.length - 1) {
					strm.write(height, "\t");
				} else {
					strm.write(height);
				}
			}

			strm.write("\n");
			pos += step;
		}

		if (isArrayed) {	//recover the old active spectrum index
			nmrSpec.curSpecIndex = activeSpec;
		}

		file.close();
		return 0;
	}

	return 1;
};

/** Add to the Env.Converters environment engine
 */
Env.Converters.push(new ASCIIMatrixColumnsConverter());

//test
ASCIIMatrixColumnsConverter.ASCIIMatrixColumnsConverterTest = function () {
	"use strict";

	var retVal = -1,
		converter = new ASCIIMatrixColumnsConverter(),
		pageItems = mainWindow.activeDocument.selection(),
		file = Dir.home() + "/ASCIIMatrixByColumns.txt";

	if (converter.wantsExport(pageItems)) {
		retVal = converter.exportItems(pageItems, file);
	}
	print(retVal + " (" + file + ")");
	return retVal;
};
