/******************************************************************************************************
Copyright (C) 2009 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided 
AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, 
MERCHANTABILITY AND FITNESSFOR A PARTICULAR PURPOSE.
*****************************************************************************************************/

// ====================================================================================================
// This file contains several functions for accessing object properties. It is needed by DBPlugin.
// Do not delete or rename the functions below! You can change the function bodies though.
// ====================================================================================================

/*globals
	Application, ASV, DBDefinition, DBPlugin, DBItem, dbPreSaveHook, DocumentWindow, mainWindow,
	MassItem, Molecule, NMRAssignments, NMRSpectrum, PageItem, Peak, Peaks, print, qsTr, Settings, SpectrumRegion, Str, Text,
	ComboBox, Dialog, GroupBox, Label, LineEdit, MessageBox, ToolButton */
/*jslint plusplus: true, indent: 4*/

function dbError(aMessage) {
	"use strict";
	MessageBox.critical(aMessage, MessageBox.Ok);
}

function dbGetItem(aUuid) {
	"use strict";
	var dw = Application.mainWindow.activeWindow();
	if (dw) {
		return dw.getItem(aUuid);
	}
	return undefined;
}

// Returns the assignments for the given molecule and nucleus as as peak list suitable for peak searches (PEAKLIST1A)
function dbGetAssignments(aMolecule, aNucleus) {
	"use strict";
	var a, assignments, atom, shift, h, smallest, s, min, max, count, peaks, l,
		shifts = [],
		shifts2 = [],
		peaklines = [];
	try {
		assignments = new NMRAssignments(aMolecule);
	} catch (e) {
		// NMRPlugin is not installed
		return "0";
	}
	// Collect chemical shifts:
	for (a = 0; a < aMolecule.atomCount; a++) {
		atom = aMolecule.atom(a);
		if (aNucleus === "13C" && atom.elementSymbol === "C") {
			shift = assignments.chemShift(a, 0);
			if (shift !== undefined) {
				shifts.push(shift);
			}
		} else if (aNucleus === "1H") {
			for (h = 1; h <= 3; h++) {
				shift = assignments.chemShift(a, h);
				if (shift !== undefined) {
					shifts.push(shift);
				}
			}
		}
	}
	//for(s in shifts) print("Shift " + s + ": min=" + shifts[s].min + " max=" + shifts[s].max);
	// Order shifts:
	while (shifts.length > 0) {
		smallest = 0;
		for (s = 1; s < shifts.length; s++) {
			if ((shifts[s].max + shifts[s].min) < (shifts[smallest].max + shifts[smallest].min)) {
				smallest = s;
			}
		}
		shifts2.push(shifts[smallest]);
		shifts.splice(smallest, 1);
	}
	shifts = shifts2;
	// Unify shifts:
	while (shifts.length > 0) {
		min = shifts[0].min;
		max = shifts[0].max;
		count = 0;
		while (shifts.length > 0 && (shifts[0].min + shifts[0].max) / 2 >= min && (shifts[0].min + shifts[0].max) / 2 <= max) {
			if (shifts[0].min < min) {
				min = shifts[0].min;
			}
			if (shifts[0].max < max) {
				max = shifts[0].max;
			}
			count++;
			shifts.splice(0, 1);
		}
		peaklines.push("{0} {1} {2} C".format((min + max) / 2, (max - min), count));
	}
	// Format result:
	peaks = "{0}".format(peaklines.length);
	for (l = 0; l < peaklines.length; l++) {
		peaks += "\n" + (l + 1) + " " + peaklines[l];
	}
	return peaks;
}

// Returns all assignments for the given molecule as text
function dbGetAssignmentsAsText(aMolecule) {
	"use strict";
	var a, assignment, atom, h,
		result = "",
		assignments = aMolecule.nmrAssignments();
		//print(JSON.stringify(assignments, null, 4))
	if (assignments) {
		for (a = 0; a < assignments.length; a++) {
			assignment = assignments[a];
			atom = aMolecule.atom(assignment.atom.index);
			h = assignment.atom.h || " ";
			result += "{0}{1}\t{2}\n".format(Str.atomIndexToString(atom.alias, atom.number,  0, false), h, JSON.stringify(assignment.shift));
		}
	}
	return result;
}

// Returns the visible spectrum region converted to ppm units
function getVisibleRegionPPM(aSpec) {
	"use strict";
	var fromX, toX, fromY, toY, unitsX, unitsY,
		zoom = aSpec.getScaleLimits(),
		peakunits = "ppm";

	// Get visible spectrum area in ppm units
	unitsX = aSpec.getProperty("axes.horizontal.units");
	if (aSpec.dimCount === 1) {
		fromX = aSpec.unitsToUnits(unitsX, peakunits, zoom.fromX, 1);
		toX = aSpec.unitsToUnits(unitsX, peakunits, zoom.toX, 1);
		return new SpectrumRegion(fromX, toX);
	}
	if (aSpec.dimCount === 2) {
		fromX = aSpec.unitsToUnits(unitsX, peakunits, zoom.fromX, 2);
		toX = aSpec.unitsToUnits(unitsX, peakunits, zoom.toX, 2);
		unitsY = aSpec.getProperty("axes.vertical.units");
		fromY = aSpec.unitsToUnits(unitsY, peakunits, zoom.fromY, 1);
		toY = aSpec.unitsToUnits(unitsY, peakunits, zoom.toY, 1);
		return new SpectrumRegion(fromX, toX, fromY, toY);
	}
	dbError("Unhandled spectrum dimension '{0}' in getVisibleRegionPPM.").format(aSpec.dimCount);
}

// Returns the total spectrum region converted to ppm units
function getTotalRegionPPM(aSpec) {
	"use strict";
	var region, zoom;
	// Remember current zoom, zoom out
	zoom = aSpec.getScaleLimits();
	aSpec.zoom();
	region = getVisibleRegionPPM(aSpec);
	// Restore zoom
	aSpec.zoom(zoom);
	return region;
}

function nmrPeakType(aNumber) {
	"use strict";
	if (aNumber === Peak.Compound) {
		return "C";
	}
	if (aNumber === Peak.Artifact) {
		return "A";
	}
	if (aNumber === Peak.Impurity) {
		return "I";
	}
	if (aNumber === Peak.Solvent) {
		return "S";
	}
	if (aNumber === Peak.CS_Ref) {
		return "R";
	}
	if (aNumber === Peak.Quantification_Ref) {
		return "Q";
	}
	return "U";
}

// Returns the peaklist for a spectrum as text suitable for db storage iff the given spectrum has dimension aDim, else "0".
function getNMRPeaks(aNMRSpectrum, aDim) {
	"use strict";
	var peaks, i, dim, peak,
		result = "0";
	if (aNMRSpectrum.dimCount !== aDim) {
		return result;
	}
	peaks = aNMRSpectrum.peaks();
	// If no peaks found, try to pick some
	if (peaks.count === 0) {
		//print("No peaks found, trying to pick...");
		peaks = new Peaks(aNMRSpectrum, getTotalRegionPPM(aNMRSpectrum));
	}
	// Build the peak list
	result = "{0}".format(peaks.count);
	for (i = 0; i < peaks.count; i++) {
		peak = peaks.at(i);
		result += "\n" + (i + 1) + " ";
		for (dim = 1; dim <= aNMRSpectrum.dimCount; dim++) {
			result += peak.delta(dim).toPrecision(9) + " " + (isNaN(peak.width(dim)) ? "0" : peak.width(dim).toPrecision(9)) + " ";
		}
		result += (isNaN(peak.intensity) ? "0" : peak.intensity.toPrecision(9)) + " " + nmrPeakType(peak.type);
	}
	return result;
}

// Returns the multiplet list for a spectrum as text suitable for db storage iff the given spectrum has dimension aDim, else "0".
// Note: only aDim=1 is supported atm.
function getNMRMultiplets(aNMRSpectrum, aDim) {
	"use strict";
	var i, m, multiplets,
		result = "0";
	if (aNMRSpectrum.dimCount === aDim) {
		multiplets = aNMRSpectrum.multiplets();
		result = "{0}".format(multiplets.count);
		for (i = 0; i < multiplets.count; i++) {
			m = multiplets.at(i);
			//print("\n    multiplet " + (i+1) + ": delta=" + m.delta + " rangeMin=" + m.rangeMin + "rangeMax=" + m.rangeMax);
			result += "\n" + (i + 1)
			       + " " + m.delta.toPrecision(9)
			       + " " + m.rangeMin.toPrecision(9)
			       + " " + m.rangeMax.toPrecision(9)
			       + " " + m.nH
			       + " " + (isNaN(m.integralValue()) ? "0" : m.integralValue().toPrecision(9))
			       + " " + m.category
			       + " " + m.jList().toString().replace("JList [", "[").replace(" ", "").replace(" ", "");
		}
	}
	return result;
}

// DEPRECATED: use getNMRpeaks() 
function getGSDPeaks(aItem, aDim) {
	"use strict";
	return getNMRPeaks(aItem, aDim);
}

// Returns the list of integrals for a spectrum as text suitable for db storage iff the given spectrum has dimension aDim, else "0".
function getIntegrals(aNMRSpectrum, aDim) {
	"use strict";
	var integrals, i, integral, dim, pos, width, absValue, relValue,
		result = "0";
	if (aNMRSpectrum.dimCount === aDim) {
		integrals = aNMRSpectrum.integrals();
		result = "{0}".format(integrals.count);
		for (i = 0; i < integrals.count; i++) {
			integral = integrals.at(i);
			result += "\n" + (i + 1) + " ";
			for (dim = 1; dim <= aNMRSpectrum.dimCount; dim++) {
				pos = (integral.rangeMax(dim) + integral.rangeMin(dim)) / 2.0;
				width = integral.rangeMax(dim) - integral.rangeMin(dim);
				result += pos.toPrecision(9) + " " + width.toPrecision(9) + " ";
			}
			absValue = integral.integralValue(1.0);
			relValue = integral.integralValue(integrals.normValue);
			result += (isNaN(absValue) ? "0" : absValue.toPrecision(9)) + " " + (isNaN(relValue) ? "0" : relValue.toPrecision(9));
		}
	}
	return result;
}

// Returns the verification result for a spectrum as an object suitable for use in db storage scripts.
// Will return a default object if any of the assumptions (exactly one molecule in document, verification has been done) are not met.
function getVerificationResult(aNMRSpectrum) {
	"use strict";
	var m, mol, verificationResults,
		dw = mainWindow.activeWindow(),
		done = false,
		// Constants
		cInvalidQuality = -10.0,
		cInvalidScore = -10.0,
		cInvalidSignificance = -1.0,
		cInvalidId = 0,
		// Construct a default result object
		result = {};
	result.moleculeId = "";
	result.id = cInvalidId;
	result.result = cInvalidScore;         // deprecated, Issue #7159
	result.quality = cInvalidQuality;
	result.score = cInvalidScore;
	result.significance = cInvalidSignificance;
	result.testsResults = [];
	result.testsResultsAsText = function () { return "0"; };
	result.toString = function () { return "Undefined VerificationResult"; };
	// Check if we have a document
	if (!dw) {
		return result;
	}
	// Check if we have a valid Verify Plugin
	if (typeof (ASV) === "undefined") {
		return result;
	}
	// See if we can get a verification result - try molecules in order of which they appear in document
	for (m = 0; m < dw.itemCount("Molecule") && !done; m++) {
		mol = new Molecule(dw.item(m, "Molecule"));
		verificationResults = ASV.getResults(aNMRSpectrum, mol);
		if (verificationResults !== undefined && verificationResults.length > 0) {
			result = verificationResults[0];
			result.moleculeId = mol.moleculeId;
			done = true;
		}
	}
	// Add on testsResults in text format for db storage
	result.testsResultsAsText = function () {
		var i, s;
		s = "{0}".format(this.testsResults.length);
		for (i = 0; i < this.testsResults.length; i++) {
			s += "\n" +  i
				+ "," + this.testsResults[i].id
				+ "," + this.testsResults[i].quality.toPrecision(4)        // = score and significance combined
				+ "," + this.testsResults[i].result.toPrecision(4)         // = score
				+ "," + this.testsResults[i].significance.toPrecision(4);
		}
		return s;
	};
	// Add on toString() utility function for debugging
	result.toString = function () {
		var i, s;
		s = String();
		if (this.moleculeId === "") {
			s = "Undefined ";
		}
		s += "VerificationResult " + this.moleculeId + " (" + this.id + "): " + this.result + " (" + this.significance + ") [";
		for (i = 0; i < this.testsResults.length; i++) {
			s += "(" + i + ":" + this.testsResults[i].id + "," + this.testsResults[i].result + "," + this.testsResults[i].significance + ")";
		}
		s += "]";
		return s;
	};
	//print(result);
	return result;
}

// Builds and returns a string suitable for databse storage
// 2011-06-07: Deprecated -- Use MassSpectrum.dbStore(plotID)
function getMassData(aItem) {
	"use strict";
	var massitem = new MassItem(aItem);
	// Begin temporary code
	if (massitem.dbStore().indexOf(" 0 0") !== -1) {
		MessageBox.warning("Warning: spectrum contains m/z values of 0.");
	}
	// End temporary code
	return massitem.dbStore();
}

// Builds and returns a search query string from a mass spectrum object
function getMassQuery(aUuid) {
	"use strict";
	var minscore, scoring, tolerance, toleranceInDa,
		massitem = new MassItem(dbGetItem(aUuid)),
		settings = new Settings("DBSearch/Settings");
	minscore = settings.value("MinHitQuality/MassItemSearch", 800) / 1000.0;
	scoring = settings.value("MS Search/Scoring", "Purity");
	tolerance = settings.value("MS Search/Tolerance", 0.5);
	toleranceInDa = settings.value("MS Search/ToleranceInDalton", true);
	return massitem.dbQuery(scoring, minscore, tolerance, toleranceInDa);
}

// Convert MassItem.ChromatogramPeakType to string
function massChromatogramPeakType(aNumber) {
	"use strict";
	if (aNumber === MassItem.ChromatogramPeakType.BB) {
		return "BB";
	}
	if (aNumber === MassItem.ChromatogramPeakType.BV) {
		return "BV";
	}
	if (aNumber === MassItem.ChromatogramPeakType.Shoulder) {
		return "Shoulder";
	}
	if (aNumber === MassItem.ChromatogramPeakType.VB) {
		return "VB";
	}
	if (aNumber === MassItem.ChromatogramPeakType.VV) {
		return "VV";
	}
	return "Unknown";
}

// Returns the chromatogram type for the given chromatogram for retention time related fields which are saved with the mass spectrum
// These chromatograms can be TIC, UV, ELSD, CNLD, UVs at different wavelengths, etc.
function dbMassChromatogramType(aMassItem, aPlotNumber) {
	"use strict";
	var msInjection, msTrace,
		plot = aMassItem.plot(aPlotNumber);

	if (!plot.valid || plot.type !== MassItem.DataTypes.Chromatogram) { // 1
		return null;
	}
	if (plot.chromatogramType === MassItem.ChromatogramType.TIC) { // 1
		return "TIC";
	}
	if (plot.chromatogramType === MassItem.ChromatogramType.Trace) {    // 7
		if (plot.injectionIndex >= 0 && plot.traceIndex >= 0) {
			msInjection = aMassItem.info.injections.getItem(plot.injectionIndex);
			msTrace = msInjection.traces.getItem(plot.traceIndex);
			if (msTrace.name.indexOf("DAD A") === 0
					|| msTrace.name.indexOf("DAD B") === 0
					|| msTrace.name.indexOf("DAD C") === 0
					|| msTrace.name.indexOf("DAD D") === 0) {
				return "UV";
			}
			if (msTrace.name.indexOf("DAD") === 0) {
				return "UVT";
			}
			if (msTrace.name.indexOf("PDA") === 0) {
				return "UVT";
			}
			if (msTrace.name.indexOf("ELSD") === 0) {
				return "ELSD";
			}
			if (msTrace.name.indexOf("CNLD") === 0) {
				return "CNLD";
			}
		}
	}
	if (plot.chromatogramType === MassItem.ChromatogramType.UV) {       // 6
		return "UV";
	}
	if (plot.chromatogramType === MassItem.ChromatogramType.UVTAbs) {   // 5
		return "UVT";
	}
	return null;
}

// Returns the field prefix for the given chromatogram for retention time related fields which are saved with the mass spectrum.
// The database definition accommodates up to 8 traces (TIC, UVT = Total UV (or DAD or PDA), ELSD, CNLD + 4 UV traces as specific wavelengths)
// Possible fields: TIC, UVT, ELSD, CNLD, UV1, UV2, UV3, UV4. 
// Up to 4 UV traces are stored invo UV1, UV2, UV3, UV4. For all but UV (=TIC, UVT, ELSD, CNLD), only use first plot.
function dbRTFieldPrefix(aMassItem, aPlotNumber) {
	"use strict";
	var p, mct,
		uvbefore = 0,
		result = dbMassChromatogramType(aMassItem, aPlotNumber);

	for (p = 0; p < aPlotNumber; p++) {
		mct = dbMassChromatogramType(aMassItem, p);
		if (mct === result) {
			if (result === "UV") {
				uvbefore++;
			} else {
				result = null;
			}
		}
	}
	if (result === "UV") {
		uvbefore++;
		if (uvbefore <= 4) {
			result += uvbefore;
		} else {
			result = null;
		}
	}
	return result;
}

// -----------------------------------------------------------------------------------------------------------------------------------------------

// Returns the first MassFunctionInfo object for the given MassItem or null if not available.
function getMassFunctionInfo(aItem) {
	"use strict";
	var plot, msinfo, injectioninfo, functionsinfo,
		massitem = new MassItem(aItem);
	plot = massitem.plotByType(MassItem.DataTypes.Spectrum);
	if (!plot) {
		return null;
	}
	//print("plot.valid = " + plot.valid);	
	if (!plot.valid) {
		return null;
	}
	msinfo = massitem.info;
	//print("msinfo.valid = " + msinfo.valid);	
	if (!msinfo.valid) {
		return null;
	}
	//print("injectionIndex = " + plot.injectionIndex);
	if (!(plot.injectionIndex >= 0 && plot.injectionIndex < msinfo.injections.count)) {
		return null;
	}
	injectioninfo = msinfo.injections.getItem(plot.injectionIndex);
	functionsinfo = injectioninfo.functions;
	//print("functionIndex = " + plot.functionIndex);
	if (!(plot.functionIndex >= 0 && plot.functionIndex < functionsinfo.count)) {
		return null;
	}
	return functionsinfo.getItem(plot.functionIndex);
}


function getMassInstrumentTypeDefault(aItem) {
	"use strict";
	var mfi = getMassFunctionInfo(aItem);
	if (!mfi) {
		return "N/A";
	}
	if (mfi.analyzerType === MassItem.MSAnalyzer.Unknown) {
		return "Unknown";
	}
	if (mfi.analyzerType === MassItem.MSAnalyzer.Constant) {
		return "Constant";
	}
	if (mfi.analyzerType === MassItem.MSAnalyzer.FTMS) {
		return "FTMS";
	}
	if (mfi.analyzerType === MassItem.MSAnalyzer.Proportional) {
		return "Proportional";
	}
	if (mfi.analyzerType === MassItem.MSAnalyzer.TOF) {
		return "TOF";
	}
	if (mfi.analyzerType === MassItem.MSAnalyzer.Other) {
		return "Other";
	}
	return "-";
}

function getMassFunctionDescriptionDefault(aItem) {
	"use strict";
	var mfi = getMassFunctionInfo(aItem);
	return mfi ? mfi.name : "N/A";
}

function getMassMsOrderDefault(aItem) {
	"use strict";
	var mfi = getMassFunctionInfo(aItem);
	return mfi ? mfi.order : "N/A";
}

function getMassPolarityDefault(aItem) {
	"use strict";
	var mfi = getMassFunctionInfo(aItem);
	if (!mfi) {
		return "N/A";
	}
	if (mfi.polarity === MassItem.FunctionPolarity.Mixed) {
		return "Mixed";
	}
	if (mfi.polarity === MassItem.FunctionPolarity.Positive) {
		return "Positive";
	}
	if (mfi.polarity === MassItem.FunctionPolarity.Negative) {
		return "Negative";
	}
	if (mfi.polarity === MassItem.FunctionPolarity.Unknown) {
		return "Unknown";
	}
	return "-";
}

// -----------------------------------------------------------------------------------------------------------------------------------------------
// Begin MassMetaDataHandler
// -----------------------------------------------------------------------------------------------------------------------------------------------

var MassMetaDataHandler = {};

MassMetaDataHandler.Field = function (aFieldName, aLabel, aDefaultValueFunction, aValidValues) {
	"use strict";
	this.fieldName = aFieldName;
	this.propertyName = "db." + aFieldName;
	this.settingsKey = "MassMetaData/" + aFieldName;
	this.label = aLabel;
	this.validValues = aValidValues;
	this.setValue = function (aItem, aPlotID, aValue) {
		aItem.setMetaData(this.propertyName + "." + aPlotID, aValue);
		new Settings().setValue(this.settingsKey, aValue);
	};
	this.getValue = function (aItem, aPlotID) {
		if (aItem.hasMetaData(this.propertyName + "." + aPlotID)) {
			return aItem.getMetaData(this.propertyName + "." + aPlotID);
		}
		return this.getDefaultValue(aItem, aPlotID);
	};
	this.getDefaultValue = function (aItem, aPlotID) {
		// 1. Use value from settings iff any
		// 2. Use value from specifed default value function iff any
		// 3. Return empty value
		return new Settings().value(this.settingsKey, aDefaultValueFunction ? aDefaultValueFunction(aItem, aPlotID) : "");
	};
};

MassMetaDataHandler.Control = function (aField, aItem, aPlotID) {
	"use strict";
	var i, val;
	this.field = aField;
	this.item = aItem;
	this.plotid = aPlotID;
	if (aField.validValues instanceof Array) {
		this.widget = new ComboBox();
		this.widget.addItem("");
		for (val = 0; val < aField.validValues.length; val++) {
			this.widget.addItem(aField.validValues[val]);
		}
		this.getWidgetData = function () {
			return this.widget.currentText;
		};
		this.setWidgetData = function (aText) {
			this.widget.currentIndex = 0;
			for (i = 0; i < this.field.validValues.length; i++) {
				if (this.field.validValues[i] === aText) {
					this.widget.currentIndex = i + 1;
				}
			}
		};
	} else {
		this.widget = new LineEdit();
		this.getWidgetData = function () {
			return this.widget.text;
		};
		this.setWidgetData = function (aText) {
			this.widget.text = aText;
		};
	}
	this.clearWidgetData = function () {
		this.setWidgetData("");
	};
	this.resetWidgetData = function () {
		this.setWidgetData(this.field.getValue(this.item, this.plotid));
	};
	this.defaultWidgetData = function () {
		this.setWidgetData(this.field.getDefaultValue(this.item, this.plotid));
	};
	this.writeValue = function () {
		this.field.setValue(this.item, this.plotid, this.getWidgetData());
	};
};

MassMetaDataHandler.fields = [];
MassMetaDataHandler.fields[0]  = new MassMetaDataHandler.Field("Instrument Name", "Instrument Name");
MassMetaDataHandler.fields[1]  = new MassMetaDataHandler.Field("Instrument Type", "Instrument Type", getMassInstrumentTypeDefault, ["Constant", "FTMS", "Proportional", "TOF", "Other", "Unknown"]);
MassMetaDataHandler.fields[2]  = new MassMetaDataHandler.Field("Function Description", "Function Description", getMassFunctionDescriptionDefault);
MassMetaDataHandler.fields[3]  = new MassMetaDataHandler.Field("Ionization Method", "Ionization Method");
MassMetaDataHandler.fields[4]  = new MassMetaDataHandler.Field("Adduct", "Adduct");
MassMetaDataHandler.fields[5]  = new MassMetaDataHandler.Field("Charge State", "Charge State");
MassMetaDataHandler.fields[6]  = new MassMetaDataHandler.Field("MS Type", "MS Type");
MassMetaDataHandler.fields[7]  = new MassMetaDataHandler.Field("MS Order", "MS Order", getMassMsOrderDefault);
MassMetaDataHandler.fields[8]  = new MassMetaDataHandler.Field("Precursor m/z", "Precursor m/z");
MassMetaDataHandler.fields[9]  = new MassMetaDataHandler.Field("Precursor Width", "Precursor Width");
MassMetaDataHandler.fields[10] = new MassMetaDataHandler.Field("Collision Energy", "Collision Energy");
MassMetaDataHandler.fields[11] = new MassMetaDataHandler.Field("Polarity", "Polarity", getMassPolarityDefault, ["Positive", "Negative", "Mixed", "Unknown"]);

// Pops up a dialog allowing user to edit meta data values
MassMetaDataHandler.showDataDialog = function (aItem, aPlotID) {
	"use strict";
	var c, f, field, label, control, tbClear, tbReset, tbDefault,
		dialog = new Dialog(),
		gbMain = new GroupBox(),
		controls = [];

	dialog.title = "Data Fields";
	gbMain.title = "Data Fields";
	dialog.add(gbMain);

	for (f = 0; f < MassMetaDataHandler.fields.length; f++) {
		field = MassMetaDataHandler.fields[f];

		label = new Label();
		label.text = field.label + ": \t";

		control = new MassMetaDataHandler.Control(field, aItem, aPlotID);
		control.resetWidgetData();
		controls.push(control);

		tbClear = new ToolButton();
		tbClear.text = "Clear";
		tbClear.clicked.connect(control, control.clearWidgetData);

		tbReset = new ToolButton();
		tbReset.text = "Reset";
		tbReset.clicked.connect(control, control.resetWidgetData);

		tbDefault = new ToolButton();
		tbDefault.text = "Default";
		tbDefault.clicked.connect(control, control.defaultWidgetData);

		gbMain.add(label, control.widget, tbClear, tbReset, tbDefault);
	}

	if (!dialog.exec()) {
		return "Error: Canceled by user.";
	}
	for (c = 0; c < controls.length; c++) {
		controls[c].writeValue();
	}
	return "Ok";
};

// -----------------------------------------------------------------------------------------------------------------------------------------------
// End MassMetaDataHandler
// -----------------------------------------------------------------------------------------------------------------------------------------------

// DEPRECATED. Still here for databases created with Mnova < 7.1.1
// Valid values for aItem.name: {"Arrow", "Ellipse", "Image", "Layout Template", "Mass Spectrum", "Molecule", "NMR Spectrum", "OLE Object", "Polygon", "Rectangle", "Text"}
function getText(aItem) {
	"use strict";
	if (aItem.name === "NMR Spectrum") {
		return new NMRSpectrum(aItem).title;
	}
	if (aItem.name === "Mass Spectrum") {
		return new MassItem(aItem).plot().description;
	}
	if (aItem.name === "Text") {
		return new Text(aItem).plainText;
	}
	return "-";
}

function getSDFAsText(aItem, aSDFTag) {
	"use strict";
	var result = new Molecule(aItem).getSDFData(aSDFTag);
	return (typeof result === "undefined")  ?  "" : result;
}

function getMassDataFromNistSDF(aItem) {
	"use strict";
	var l, lines, result,
		sdftext = new Molecule(aItem).getSDFData("MASS SPECTRAL PEAKS");
	if (sdftext === undefined) {
		return "0 0";
	}
	//print("sdftext: '" + sdftext + "'");
	lines = sdftext.split("\n");
	result = "1 " + lines.length + "\n<null>\n";
	for (l = 0; l < lines.length; l++) {
		result += "{0} {1}\n".format(l + 1, lines[l]);
	}
	return result;
}

function dbMergePageNotes(aPageNumber, aNotes) {
	"use strict";
	//print("mergePageNotes(" + aPageNumber + ",'" + aNotes + "')");
	var page,
		doc = new DocumentWindow(Application.mainWindow.activeWindow());
	if (aPageNumber < 0 || aPageNumber >= doc.pageCount()) {
		return;
	}
	page = doc.page(aPageNumber);
	if (page.notes.indexOf(aNotes) < 0) {
		if (page.notes.length > 0) {
			page.notes += "\n";
		}
		page.notes += aNotes;
	}
}

// DEPRECATED. Still here for databases created with Mnova < 7.1.1
function getPageNotes(aItemID) {
	"use strict";
	var p, page, i,
		doc = new DocumentWindow(Application.mainWindow.activeWindow());
	for (p = 0; p < doc.pageCount(); p++) {
		page = doc.page(p);
		for (i = 0; i < page.itemCount(); i++) {
			if (page.item(i).uuid === aItemID) {
				return page.notes;
			}
		}
	}
	return "";
}

// DEPRECATED. Still here for databases created with Mnova < 7.1.1
function mergePageNotes(aItemID, aNotes) {
	"use strict";
	//print("mergePageNotes(" + aItemID + ",'" + aNotes + "')");
	var p, page, i,
		doc = new DocumentWindow(Application.mainWindow.activeWindow());
	for (p = 0; p < doc.pageCount(); p++) {
		page = doc.page(p);
		for (i = 0; i < page.itemCount(); i++) {
			if (page.item(i).uuid === aItemID) {
				dbMergePageNotes(p, aNotes);
				return;
			}
		}
	}
}

// Finds a text item by type and subtype and returns its contents
function getNamedTextField(aType, aSubtype) {
	"use strict";
	var i, item,
		dw = new DocumentWindow(mainWindow.activeWindow());
	for (i = 0; i < dw.itemCount(); i++) {
		item = dw.item(i);
		if (item.name === "Text" && item.type === aType && item.subtype === aSubtype) {
			item.update();
			return new Text(item).plainText;
		}
	}
	return "-";
}

// Returns true iff aX is between aY and aZ
function isBetween(aX, aY, aZ) {
	"use strict";
	return ((aY <= aX && aX <= aZ) || (aZ <= aX && aX <= aY));
}

// Returns true iff [aA,aB] is completetely included in [aC,aD]
function isInside(aA, aB, aC, aD) {
	"use strict";
	return isBetween(aA, aC, aD) && isBetween(aB, aC, aD);
}

// -----------------------------------------------------------------------------------------------------------------------------------------------

// Returns the metadata for the given mass spectrum as an associative array [Field Name] => [value] 
function dbGetMassSpecMetaData(aItem, aPlotID) {
	"use strict";
	var f, field,
		fields = {},
		settings = new Settings("Database");
	if (!aItem.hasMetaData("db.NoDialog") && settings.value("Show Data Dialogs", true)) {
		MassMetaDataHandler.showDataDialog(aItem, aPlotID);
	}
	for (f = 0; f < MassMetaDataHandler.fields.length; f++) {
		field = MassMetaDataHandler.fields[f];
		fields[field.fieldName] = field.getValue(aItem, aPlotID);
	}
	return fields;
}

// -----------------------------------------------------------------------------------------------------------------------------------------------

// Add some db fields common to all items to the given db item.
function dbAddCommonFields(aDBItem, aPageItem) {
	"use strict";
	var i, p, page, pageItem;
	//print(aPageItem.constructor.name);
	if (aPageItem) {                                            // mgo 2014-04-02: Bug #7172 aPageItem may be null (for page notes)
		pageItem = new PageItem(aPageItem);                 // mgo 2014-04-02: Bug #7166 (Molecule.name hides PageItem.name) is fixed but better be safe
		aDBItem.addField("Uuid", pageItem.uuid);
		aDBItem.addField("LinkID", pageItem.linkId);
		aDBItem.addField("Type", pageItem.name);
		aDBItem.addField("Preview", DBPlugin.getPreview(pageItem));
		//aDBItem.addField("Svg", DBPlugin.getSVG(pageItem));
		//aDBItem.addField("Notes", getPageNotes(pageItem.uuid));
		// Get the page number:
		for (p = 0; p < mainWindow.activeDocument.pageCount(); p++) {
			page = mainWindow.activeDocument.page(p);
			for (i = 0; i < page.itemCount(); i++) {
				if (page.item(i).uuid === pageItem.uuid) {
					aDBItem.addField("Page", p + 1);
					aDBItem.addField("PageID", page.uuid);
				}
			}
		}
	}
	aDBItem.addField("Host", DBPlugin.client.name);
	aDBItem.addField("Document", mainWindow.activeWindow().name);
	aDBItem.addField("DocumentID", mainWindow.activeWindow().uuid);
	aDBItem.addField("User", DBPlugin.client.user);
	aDBItem.addField("Created", "NOW");
	aDBItem.addField("Modified", "NOW");
	aDBItem.addField("Mnova Version", Application.version.full);
}

// Add custom fields from the given db definition to the given db item.
function dbAddCustomFields(aDBItem, aPageItem, aDB) {
	"use strict";
	var f, contentDef, item,
		fields = aDB.fields(aDBItem.type);

	function tryEval(aCode) {
		try {
			return eval(aCode);
		} catch (e) {
			return "Error: " + e.name + " - " + e.message;
		}
	}

        //print("DBItem type: " + dbItems[i].type);
	for (f = 0; f < fields.length; ++f) {
		if (!aDBItem.hasField(fields[f])) {
			contentDef = Str.trimStr(aDB.contentDef(aDBItem.type, fields[f]));
			//print(contentDef);
			if (contentDef.indexOf("=") === 0) {
				item = aPageItem;
				aDBItem.addField(fields[f], tryEval(contentDef.substr(1)));
			}
		}
	}
}

function dbMoleculeToDBItem(aDBClient, aDB, aMolecule, aShowErrors) {
	"use strict";
	var dbItem = new DBItem("Molecule");
	dbItem.addField("Name", aMolecule.molName);
	dbItem.addField("Label", aMolecule.label);
	dbItem.addField("Description", aMolecule.description);
	dbItem.addField("Aliases", aMolecule.aliases.join("\n"));
	dbItem.addField("Atoms", aMolecule.atomCount);
	dbItem.addField("Bonds", aMolecule.bondCount);
	dbItem.addField("MoleculeId", aMolecule.moleculeId);
	dbItem.addField("CHEMICAL STRUCTURE", aMolecule.getMolfile());
	dbItem.addField("Molecular Formula", aMolecule.molecularFormula());
	dbItem.addField("Average Mass", aMolecule.averageMass());
	dbItem.addField("Monoisotopic Mass", aMolecule.monoisotopicMass());
	dbItem.addField("DbaId", aMolecule.getParam("DBAID"));
	dbItem.addField("Assignments 1H", dbGetAssignments(aMolecule, "1H"));
	dbItem.addField("Assignments 13C", dbGetAssignments(aMolecule, "13C"));
	dbAddCommonFields(dbItem, aMolecule);
	dbAddCustomFields(dbItem, aMolecule, aDB);
	return dbItem;
}

function dbNMRSpectrumToDBItem(aDBClient, aDB, aSpectrum, aShowErrors) {
	"use strict";
	var i,
		paramNames = aSpectrum.paramNames(),
		dbItem = new DBItem("NMR Spectrum");
	dbItem.addField("Description", aSpectrum.title);
	for (i = 0; i < paramNames.length; i++) {
		if (aDB.hasField("NMR Spectrum", paramNames[i])) {
			dbItem.addField(paramNames[i], aSpectrum.getParam(paramNames[i]));
		}
	}
	dbItem.addField("Peaks1D", getNMRPeaks(aSpectrum, 1));
	dbItem.addField("Multiplets1D", getNMRMultiplets(aSpectrum, 1));
	dbItem.addField("Integrals1D", getIntegrals(aSpectrum, 1));
	dbItem.addField("Peaks2D", getNMRPeaks(aSpectrum, 2));
	dbItem.addField("Integrals2D", getIntegrals(aSpectrum, 2));
	dbItem.addField("Spectrum Title", aSpectrum.title);
	dbItem.addField("Spectrum Comment", aSpectrum.comment);
	dbItem.addField("Experiment Type", aSpectrum.experimentType);
	dbItem.addField("Original Format", aSpectrum.originalFormat);
	dbItem.addField("Solvent", aSpectrum.solvent);
	dbAddCommonFields(dbItem, aSpectrum);
	dbAddCustomFields(dbItem, aSpectrum, aDB);
	return dbItem;
}

function dbMassItemToDBItems(aDBClient, aDB, aMassItem, aShowErrors) {
	"use strict";

	function findBestChromPeak(aMassSpectrum, aMassChromatogram) {
		// Find corresponding chromatogram peak (= tallest peak within mass spectrum ranges)
		if (aMassChromatogram.peaks === undefined) {
			return undefined;
		}
		var bestChromPeak, p, currPeak, ft, foreRange;
		for (p = 0; p < aMassChromatogram.peaks.length; p++) {
			currPeak = aMassChromatogram.peaks[p];
			//print("  currPeak: " + currPeak.center.x);
			for (ft = 0; ft < aMassSpectrum.times.foreTimes.length; ft++) {
				foreRange = aMassSpectrum.times.foreTimes[ft];
				//print("   ft=" + ft + " [" + foreRange.from + ".." + foreRange.to + "]");
				if (isBetween(currPeak.center.x, foreRange.from, foreRange.to) || isInside(foreRange.from, foreRange.to, currPeak.start.x, currPeak.end.x)) {
					if (bestChromPeak === undefined || currPeak.height > bestChromPeak.height) {
						//print("    IsBetween");
						bestChromPeak = currPeak;
					}
				}
			}
		}
		return bestChromPeak;
	}

	var s, spec, c, chrom, prefix, bestChromPeak, p, currPeak, ft, foreRange, dbItem, chromPeaksTotalHeight, chromPeaksTotalArea,
		dbItems = [];

	// Iterate through plots, building one dbItem for each mass spectrum
	for (s = 0; s < aMassItem.plotsCount; s++) {
		spec = aMassItem.plot(s);
		if (spec.valid && spec.visible && spec.type === MassItem.DataTypes.Spectrum) {
			//print("Spectrum " + s);
			dbItem = new DBItem("Mass Spectrum");
			dbItem.addField("Description", spec.description);
			dbItem.addField("Dataset FileName", aMassItem.datasetFileName);
			dbItem.addField("PlotID", spec.id);
			dbItem.addField("PlotNumber", s);
			dbItem.addField("Mass Data", aMassItem.dbStore(spec.id));
			// Iterate through chromatogram plots for RT fields
			for (c = 0; c < aMassItem.plotsCount; c++) {
				chrom = aMassItem.plot(c);
				prefix = dbRTFieldPrefix(aMassItem, c);
				if (chrom.valid && chrom.visible && chrom.type === MassItem.DataTypes.Chromatogram && aDB.hasField("Mass Spectrum", prefix + "_RT")) {
					//print(" Chromatogram " + c);
					// Find corresponding chromatogram peak (= tallest peak within mass spectrum ranges)
					bestChromPeak = findBestChromPeak(spec, chrom);
					if (bestChromPeak !== undefined) {
						// Calculate total Height and Area of all chromatogram peaks
						chromPeaksTotalHeight = 0.0;
						chromPeaksTotalArea = 0.0;
						for (p = 0; p < chrom.peaks.length; p++) {
							chromPeaksTotalHeight += chrom.peaks[p].height;
							chromPeaksTotalArea += chrom.peaks[p].area;
						}
						// Add fields for corresonding chromatogram peak
						dbItem.addField(prefix + "_RT", bestChromPeak.center.x);
						dbItem.addField(prefix + "_Scan", bestChromPeak.scan);
						dbItem.addField(prefix + "_Type", massChromatogramPeakType(bestChromPeak.type));
						dbItem.addField(prefix + "_Height", bestChromPeak.height);
						dbItem.addField(prefix + "_Area", bestChromPeak.area);
						if (chromPeaksTotalHeight > 0.0) {
							dbItem.addField(prefix + "_HeightPercent", bestChromPeak.height * 100.0 / chromPeaksTotalHeight);
						}
						if (chromPeaksTotalArea > 0.0) {
							dbItem.addField(prefix + "_AreaPercent", bestChromPeak.area * 100.0 / chromPeaksTotalArea);
						}
						dbItem.addField(prefix + "_StartTime", bestChromPeak.start.x);
						dbItem.addField(prefix + "_EndTime", bestChromPeak.end.x);
					}
				}
			}
			dbAddCommonFields(dbItem, aMassItem);
			dbAddCustomFields(dbItem, aMassItem, aDB);
			dbItems.push(dbItem);
		}
	}
	return dbItems;
}

// Convert a Page Item to an array of DB items.
function dbPageItemToDBItems(aDBClient, aDB, aPageItem, aShowErrors) {
	"use strict";
	var dbItem, massDBItems, i,
		dbItems = [];

	if (aPageItem.name === "Molecule") {
		dbItems.push(dbMoleculeToDBItem(aDBClient, aDB, new Molecule(aPageItem), aShowErrors));

	} else if (aPageItem.name === "NMR Spectrum" && aDB.hasItemType("NMR Spectrum")) {
		dbItems.push(dbNMRSpectrumToDBItem(aDBClient, aDB, new NMRSpectrum(aPageItem), aShowErrors));

	} else if (aPageItem.name === "Mass Spectrum" && aDB.hasItemType("Mass Spectrum")) {
		massDBItems = dbMassItemToDBItems(aDBClient, aDB, new MassItem(aPageItem), aShowErrors);
		for (i = 0; i < massDBItems.length; i++) {
			dbItems.push(massDBItems[i]);
		}

	} else if (aPageItem.name === "Text" && aDB.hasItemType("Mnova")) {
		dbItem = new DBItem("Mnova");
		dbItem.addField("Description", new Text(aPageItem).plainText);
		dbAddCommonFields(dbItem, aPageItem);
		dbAddCustomFields(dbItem, aPageItem, aDB);
		dbItems.push(dbItem);

	} else if (aDB.hasItemType("Mnova")) {
		// Anything else: images, ... 
		dbItem = new DBItem("Mnova");
		dbItem.addField("Description", aPageItem.name);
		dbAddCommonFields(dbItem, aPageItem);
		dbAddCustomFields(dbItem, aPageItem, aDB);
		dbItems.push(dbItem);
	}
	//print(dbItems);
	return dbItems;
}

// Convert the given page items to an array of db items:
function dbPageItemsToDBItems(aDBClient, aDB, aPageItems, aShowErrors) {
	"use strict";
	var pi, dbItems, i,
		result = [];
	for (pi = 0; pi < aPageItems.length; pi++) {
		dbItems = dbPageItemToDBItems(aDBClient, aDB, aPageItems[pi], aShowErrors);
		for (i = 0; i < dbItems.length; i++) {
			result.push(dbItems[i]);
		}
	}
	return result;
}


// This function is called when saving to database. It returns an array of DBItems built from the current document.
// Parameters: aDBClient - DBClient object which is connected to the server we want to save to
//             aDB - DBDefinition object for the database where items will be saved to
function dbMakeDBItems(aDBClient, aDB, aShowErrors, aInteractive) {
	"use strict";
	var i, p, page, pageItem, dbItem, dbItems, metaData, fieldName,
		dw = mainWindow.activeWindow(),
		pageItems = [];

	if (!dw) {
		return [];
	}

	//print("dbMakeDBItems script [Beg]");
	// Collect page items from current document:
	for (i = 0; i < dw.getMolTableItemCount(); i++) {
		pageItems.push(dw.getMolTableItem(i));
	}
	for (i = 0; i < dw.itemCount(); i++) {
		if (dw.item(i).name !== "Molecule") {
			pageItems.push(dw.item(i));
		}
	}

	print("{0} Page items.".format(pageItems.length));
	//for (i = 0; i < pageItems.length; i++) {                                   
	//	print("PageItem " + i + ": UUID=" + pageItems[i].uuid + " Name=" + pageItems[i].name);   
	//}

	// Build db items from page items:
	mainWindow.setBusy("Preparing...");
	dbItems = dbPageItemsToDBItems(aDBClient, aDB, pageItems, aShowErrors);
	mainWindow.endBusy();

	// Add page notes
	if (aDB.hasItemType("Page Note")) {
		for (p = 0; p < mainWindow.activeDocument.pageCount(); p++) {
			page = mainWindow.activeDocument.page(p);
			if (page.notes) {
				dbItem = new DBItem("Page Note");
				dbItem.addField("Type", "Page Note");
				dbItem.addField("Description", page.notes);
				dbItem.addField("Text", page.notes);
				dbItem.addField("Page", p + 1);
				dbItem.addField("PageID", page.uuid);
				dbItem.addField("Uuid", page.uuid);
				dbAddCommonFields(dbItem, null);
				dbItems.push(dbItem);
			}
		}
	}

	//for (i = 0; i < dbItems.length; i++) {
	//		print("Item " + i + ":\n" + dbItems[i].toString());
	//}

	// Show the item selection dialog
	if (dbItems.length > 0 && aInteractive) {
		dbItems = DBPlugin.selectDBItems(aDB, dbItems);
	}

	//print("" + dbItems.length + " db items. [1]");

	// Call Pre-Save hook if it exists
	if (typeof dbPreSaveHook === 'function') {
		dbPreSaveHook(aDBClient, aDB, 0, dbItems, aShowErrors, aInteractive);
	}

	//print("" + dbItems.length + " db items. [2]");

	// Add custom field contents
	for (i = 0; i < dbItems.length; i++) {
		pageItem = dbGetItem(dbItems[i].field("Uuid"));
		if (pageItem) {
			// Show metadata dialogs for MassItem plots
			if (dbItems[i].type === "Mass Spectrum") {
				metaData = dbGetMassSpecMetaData(pageItem, dbItems[i].field("PlotId"));
				for (fieldName in metaData) {
					if (metaData.hasOwnProperty(fieldName)) {
						dbItems[i].setField(fieldName, metaData[fieldName]);
					}
				}
			}
		}
	}

	//print("dbMakeDBItems script [End]");
	return dbItems;
}


// ---------------------------------------------------------------------------------------------------------------------
// dbPrivate contains helper functions which are not used anywhere outside this script.
// ---------------------------------------------------------------------------------------------------------------------

var dbPrivate = {};

// ---------------------------------------------------------------------------------------------------------------------
// dbTest contains functions for testing and debugging which are not used anywhere outside this script.
// ---------------------------------------------------------------------------------------------------------------------

var dbTest = {};

// Build db items from the current current and print the result to the debug console.
dbTest.testMakeDBItems = function () {
	"use strict";
	var i,
		items = dbMakeDBItems(DBPlugin.client, DBDefinition.defaultDefinition("Test Database"), true, true);
	for (i = 0; i < items.length; i++) {
		print("Item " + i + ":\n" + items[i].toString());
	}
};

dbTest.testMolecule = function (aMolecule) {
	"use strict";
	var i, paramNames, sdfNames;
	//print(item);
	print("    molFile = " + aMolecule.getMolfile());
	print("    molecularFormula = " + aMolecule.molecularFormula());
	print("    molecularWeight = " + aMolecule.molecularWeight());
	paramNames = aMolecule.paramNames();
	print("    " + paramNames.length +  " parameters:");
	for (i = 0; i < paramNames.length; i++) {
		print("        " + paramNames[i] + ": " + aMolecule.getParam(paramNames[i]));
	}
	sdfNames = aMolecule.sdfNames();
	print("    " + sdfNames.length +  " sdf tags:");
	for (i = 0; i < sdfNames.length; i++) {
		print("        " + sdfNames[i] + ": " + aMolecule.getSDFData(sdfNames[i]));
	}
	print("    Assignmnents: " + dbGetAssignmentsAsText(aMolecule));
	print("    Assignments 1H: " + dbGetAssignments(aMolecule, "1H"));
	print("    Assignments 13C: " + dbGetAssignments(aMolecule, "13C"));
};

dbTest.testNMRSpectrum = function (aSpectrum) {
	"use strict";
	var i,
		visibleRegion = getVisibleRegionPPM(aSpectrum),
		totalRegion = getTotalRegionPPM(aSpectrum),
		paramNames = aSpectrum.paramNames();
	print("    visible region Dim1: from=" + visibleRegion.from(1) + " to=" + visibleRegion.to(1) + " Dim2: from=" + visibleRegion.from(2) + " to=" + visibleRegion.to(2));
	print("    total region Dim1: from=" + totalRegion.from(1) + " to=" + totalRegion.to(1) + " Dim2: from=" + totalRegion.from(2) + " to=" + totalRegion.to(2));
	print("    " + paramNames.length +  " parameters.");
	for (i = 0; i < paramNames.length; i++) {
		print("      " + paramNames[i] + ": " + aSpectrum.getParam(paramNames[i]));
	}
	print("    dimCount = " + aSpectrum.dimCount);
	print("    nmrPeaks1D = " + getNMRPeaks(aSpectrum, 1));
	print("    nmrPeaks2D = " + getNMRPeaks(aSpectrum, 2));
	print("    nmrMultiplets = " + getNMRMultiplets(aSpectrum, 1));
	print("    nmrOriginalFormat = " + aSpectrum.originalFormat);
	print("    integrals1D = " + getIntegrals(aSpectrum, 1));
	print("    integrals2D = " + getIntegrals(aSpectrum, 2));
	print("    verification result = " + getVerificationResult(aSpectrum).testsResultsAsText());
};

dbTest.testMassItem = function (aMassItem) {
	"use strict";
	var i, msInjection, f, msFunction, t, msTrace, p, plot, field,
		ms = aMassItem;
	print("    Dataset File = " + ms.datasetFileName);
	print("    Info valid = " + ms.info.valid);
	if (ms.info.valid) {
		print("    Injections count = " + ms.info.injections.count);
		for (i = 0; i < ms.info.injections.count; i++) {
			msInjection = ms.info.injections.getItem(i);
			print("      Injection " + i + " Name = '" + msInjection.name + "'");
			print("        Functions count = " + msInjection.functions.count);
			for (f = 0; f < msInjection.functions.count; f++) {
				print("          Function " + f + " Name = '" + msInjection.functions.getItem(f).name + "'");
			}
			print("        Traces count = " + msInjection.traces.count);
			for (t = 0; t < msInjection.traces.count; t++) {
				print("          Trace " + t + " Name = '" + msInjection.traces.getItem(t).name + "'");
			}
		}
	}
	print("    " + MassMetaDataHandler.fields.length + " fields:");
	for (f = 0; f < MassMetaDataHandler.fields.length; f++) {
		field = MassMetaDataHandler.fields[f];
		print("      " + field.label + " = '" + aMassItem.getMetaData(field.propertyName) + "' (Default = '" + field.getDefaultValue(aMassItem, field) + "')");
	}
	print("    " + ms.plotsCount + " plots:");
	for (p = 0; p < ms.plotsCount; p++) {
		plot = ms.plot(p);
		print("        Plot " + p + " Type=" + plot.type + " PlotId =" + plot.id + " Valid=" + plot.valid);
		print("          injectionIndex=" + plot.injectionIndex  + "  functionIndex=" + plot.functionIndex + " traceIndex=" + plot.traceIndex);
		if (ms.info.valid && plot.injectionIndex >= 0) {
			msInjection = ms.info.injections.getItem(plot.injectionIndex);
			print("          Injection " + plot.injectionIndex + ": " + msInjection.name);
			if (plot.functionIndex >= 0) {
				msFunction = msInjection.functions.getItem(plot.functionIndex);
				print("          Function " + plot.functionIndex + ": " + msFunction.name);
			}
			if (plot.traceIndex >= 0) {
				msTrace = msInjection.traces.getItem(plot.traceIndex);
				print("          Trace " + plot.traceIndex + ": " + msTrace.name);
			}
		}
		if (plot.type === MassItem.DataTypes.Chromatogram) {
			// plot type === 1
			print("          Chromatogram " + " Description='" + plot.description + "'");
			print("          Chromatogram Type: " + dbMassChromatogramType(ms, p));
			print("          RT Prefix: " + dbRTFieldPrefix(ms, p));
			//print(plot.peaks);
		} else if (plot.type === MassItem.DataTypes.Spectrum) {
			// plot type === 2
			print("          Mass Spectrum " + " Description='" + plot.description + "'");
			print("          Chromatogram Type: " + dbMassChromatogramType(ms, p)); // Should be null
			print("          RT Prefix: " + dbRTFieldPrefix(ms, p));                // Should be null
			print("           foreTimes:");
			for (t = 0; t < plot.times.foreTimes.length; t++) {
				print("             " + t + ": " + plot.times.foreTimes[t].from + ".." + plot.times.foreTimes[t].to);
			}
			print("           backTimes:");
			for (t = 0; t < plot.times.backTimes.length; t++) {
				print("             " + t + ": " + plot.times.backTimes[t].from + ".." + plot.times.backTimes[t].to);
			}
		} else {
			print("           Unknown mass plot type" + p + " PlotId = '" + plot.id[p] + "' Description='" + plot.description + "' valid=" + plot.valid);
		}
	}
};


// Print information about the page item with the given uuid to the debug console.
dbTest.testPageItem = function (aUuid) {
	"use strict";
	var i, molecule, paramNames, sdfNames, spectrum, region, ms, msInjection, msFunction, msTrace, f, t, field, p, plot,
		item = dbGetItem(aUuid);
	//print(item);
	if (item.name === "Molecule") {
		this.testMolecule(new Molecule(item));
	} else if (item.name === "NMR Spectrum") {
		this.testNMRSpectrum(new NMRSpectrum(item));
	} else if (item.name === "Mass Spectrum") {
		this.testMassItem(new MassItem(item));
	}
	print("    Text = " + getText(item));
	print("    Notes = " + getPageNotes(item));
};

// Print information about page items in the current document to the debug console
dbTest.testPageItemsAll = function () {
	"use strict";
	var i,
		dw = mainWindow.activeWindow();
	for (i = 0; i < dw.itemCount(); i++) {
		print("  Item " + i + ": " + dw.item(i) + " Name='" + dw.item(i).name + "' Type='" + dw.item(i).type + "' Subtype='" + dw.item(i).subtype + "'");
		this.testPageItem(dw.item(i).uuid);
	}
	print(dw.itemCount() + " items on " + dw.pageCount() + " pages.");
};

// Print information about page items in the current document to the debug console
dbTest.testPageItemsSelection = function () {
	"use strict";
	var i, items,
		dw = mainWindow.activeWindow();
	items = dw.selection();
	for (i = 0; i < items.length; i++) {
		print("  Item " + i + ": " + items[i] + " Name='" + items[i].name + "' Type='" + items[i].type + "' Subtype='" + items[i].subtype + "'");
		this.testPageItem(items[i].uuid);
	}
	print(items.length + " items in selection.");
};

// Print a short listing of all items in to the current document to the debug console
dbTest.printItems = function () {
	"use strict";
	var i,
		dw = mainWindow.activeWindow();
	if (!dw) {
		return;
	}
	for (i = 0; i < dw.itemCount(); i++) {
		print(" Item " + i + ": " + dw.item(i));
	}
	print(dw.itemCount() + " items on " + dw.pageCount() + " pages.");
};
/*
dbTest.printEval = function(code)
{
	print(code + " = " + eval(code));
}
dbTest.printConstants = function() 
{
	var i, peaktypes, gsd
	var peaktypes = ['Compound', 'Artifact', 'Impurity', 'Solvent', 'CS_Ref', 'Quantification_Ref'];
	for (i in peaktypes) printEval("Peak." + peaktypes[i]);
	var gsdpeaktypes = ['Compound', 'Artifact', 'Impurity', 'Solvent', 'SReference', 'QReference'];
	for (i in gsdpeaktypes) printEval("GSDPeak.Types." + gsdpeaktypes[i]);
	var gsdpeakflags = ['C13Satellite', 'Hidden', 'Labile', 'None', 'Rotational', 'Weak'];
	for (i in gsdpeakflags) printEval("GSDPeak.Flags." + gsdpeakflags[i]);
}
*/

// ---------------------------------------------------------------------------------------------------------------------
// End of file.

