/******************************************************************************************************
Copyright (C) 2013 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*****************************************************************************************************/

/*globals BaseConverter, NMRSpectrum, print, File, TextStream, Str, Env, Application*/
/*jslint plusplus: true, indent: 4*/

/** Default constructor of the converter
 * Defines the file filter
*/
function IntegralSeriesConverter() {
	"use strict";
	BaseConverter.call(this);
	this.exportFileFilter = "1D Integral Series (*.txt)"; //!< It will override any other script converter with the same exportFileFilter
}
IntegralSeriesConverter.prototype = new BaseConverter();

/** Can export 1D NMRSpectrums with integrals
 */
IntegralSeriesConverter.prototype.wantsExport = function (aPageItems) {
	"use strict";
	var i, nmrSpc;

	for (i = 0; i < aPageItems.length; i++) {
		nmrSpc = new NMRSpectrum(aPageItems[i]);
		if (nmrSpc.isValid()) {
			if (nmrSpc.dimCount === 1 && nmrSpc.integrals().count !== 0) {
				return true;
			}
		}
	}
	return false;
};

/** Gets the integral list
 */
IntegralSeriesConverter.prototype.integralList = function (aNMRSpec) {
	"use strict";
	return aNMRSpec.integrals();
};

/** Writes single integral
 */
IntegralSeriesConverter.prototype.writeSingleIntegral = function (aIntegral, aNormValue, aStrm) {
	"use strict";
	var min = aIntegral.rangeMin(1),
		max = aIntegral.rangeMax(1),
		absValue = aIntegral.integralValue(1.0),
		normValue = aIntegral.integralValue(aNormValue);
	aStrm.write(max, "\t", min, "\t", normValue, "\t", absValue, "\t");
};

/** Writes the file
 */
IntegralSeriesConverter.prototype.exportItems = function (aPageItems, aFilename) {
	"use strict";

	print(aPageItems);
	print(aFilename);

	var counter, strm, i, nmrSpc, originalSpcIndex, spcCount, s, integralList, j, integral,
		file = new File(aFilename);

	if (file.open(File.WriteOnly)) {
		counter = 1;
		strm = new TextStream(file);
		for (i = 0; i < aPageItems.length; i++) {
			if (aPageItems[i].name === "NMR Spectrum") {
				nmrSpc = new NMRSpectrum(aPageItems[i]);
				if (nmrSpc.dimCount === 1) {
					originalSpcIndex = nmrSpc.curSpecIndex;
					spcCount = nmrSpc.specCount;
					for (s = 0; s < spcCount; s++) { //it could be an arrayed spectrum
						nmrSpc.curSpecIndex = s;
						strm.write(counter, "\t", Str.toOneWord(nmrSpc.title), "\t"); //toOneWord defined in string.qs
						integralList = this.integralList(nmrSpc);
						if (integralList.count) {
							for (j = 0; j < integralList.count; j++) {
								integral = integralList.at(j);
								this.writeSingleIntegral(integral, integralList.normValue, strm);
							}
						}
						strm.write("\n");
						counter++;
					}
					strm.write("\n");
					nmrSpc.curSpecIndex = originalSpcIndex;
				}
			}
		}
		file.close();
		return 0;
	}
	return 1;
};

/** Add to the Env.Converters environment engine
 */
Env.Converters.push(new IntegralSeriesConverter());

//test
function integralSeriesConverterTest() {
	"use strict";
	var retVal = -1,
		integralSConv = new IntegralSeriesConverter(),
		pageItems = Application.mainWindow.activeDocument.selection();

	if (integralSConv.wantsExport(pageItems)) {
		retVal = integralSConv.exportItems(pageItems, "/home/isaac/integrals.txt");
	}
	return retVal;
}

/** Multiplets Integrals series converter
 */
function MultipletIntegralSeriesConverter() {
	"use strict";
	IntegralSeriesConverter.call(this);
	this.exportFileFilter = "Multiplet Integral Series (*.txt)";
}

MultipletIntegralSeriesConverter.prototype = new IntegralSeriesConverter();

/** Override the base class method
 */
MultipletIntegralSeriesConverter.prototype.integralList = function (aNMRSpec) {
	"use strict";
	return aNMRSpec.multiplets();
};

/** Writes single integral
 */
MultipletIntegralSeriesConverter.prototype.writeSingleIntegral = function (aIntegral, aNormValue, aStrm) {
	"use strict";
	var min = aIntegral.rangeMin,
		max = aIntegral.rangeMax,
		absValue = aIntegral.integralValue(1.0),
		normValue = aIntegral.integralValue(aNormValue);
	aStrm.write(max, "\t", min, "\t", normValue, "\t", absValue, "\t");
};

/** Can export NMRSpectrums with integrals
 */
MultipletIntegralSeriesConverter.prototype.wantsExport = function (aPageItems) {
	"use strict";
	var i, nmrSpc;
	for (i = 0; i < aPageItems.length; i++) {
		nmrSpc = new NMRSpectrum(aPageItems[i]);
		if (nmrSpc.isValid()) {
			return nmrSpc.multiplets().count > 0;
		}
	}
	return false;
};

Env.Converters.push(new MultipletIntegralSeriesConverter());
