/******************************************************************************************************
Copyright (C) 2013 Mestrelab Research S.L. All rights reserved.

This file is part of the MNova scripting toolkit.

Authorized users of MNova Software may use this file freely, but this file is provided AS IS
with NO WARRANTY OF ANY KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE.
*****************************************************************************************************/

/*globals FileDialog, settings, qsTr, Dir, File, dbOpenDatabase, serialization, MessageBox, Application, ProgressDialog, TextStream, Resumable, Str, DBPlugin, dbMakeDBItems, print, MnUi*/
/*jslint plusplus: true, indent: 4*/

function dbImportSDFile() {
	"use strict";
	var dbname, inFile, inFileName, inStream, outStream, progress, line, resumable, startTime,
		tempFile, tempFileName, tempFileNamePrefix, buffer, endConnectionTableWritten, ok, numCompounds, molCount, molCountString, successCount,
		dbDef, dw, dbItems, cid, l,
		progressLabel = qsTr("Reading file ...\n{0} Molecules\n{1} of {2} characters read"),
	// Save value of FileHistory setting
		oldFileHistory = serialization.fileHistory,
	// Settings keys
		inDirSettingsKey = "Database/Import SDFile/Input Directory",
		resumeSettingsKey = "Database/Import SDFile/Resume";

	// Prompt for file to import
	inFileName = FileDialog.getOpenFileName("SDfile (*.sdf);;All files (*.*)", qsTr("Select File"), settings.value(inDirSettingsKey, Dir.home()));
	if (!inFileName.length) {
		return;
	}
	settings.setValue(inDirSettingsKey, File.absDirPath(inFileName));

	// Choose database
	dbname = dbOpenDatabase();
	if (!dbname) {
		return;
	}

	// Check that file is readable
	serialization.fileHistory = false;
	inFile = new File(inFileName);
	if (!inFile.open(File.ReadOnly)) {
		MessageBox.critical(qsTr("Could not open file '{0}'.").format(inFileName));
		return;
	}

	// First run: Count compounds
	Application.mainWindow.setBusy(qsTr("Working..."));
	progress = new ProgressDialog();
	progress.setRange(0, inFile.size);
	progress.value = 0;
	progress.labelText = progressLabel.format(0, 0, inFile.size);
	progress.show();
	inStream = new TextStream(inFile);
	numCompounds = 0;
	while (!inStream.atEnd() && !progress.wasCanceled) {
		line = inStream.readLine();
		//print("'" + line + "'");
		if (line === "$$$$") {
			numCompounds++;
			progress.value = inStream.pos;
			progress.labelText = progressLabel.format(numCompounds + 1, inStream.pos, inFile.size);
			Application.mainWindow.processEvents();
		}
	}
	inFile.close();
	Application.mainWindow.endBusy();

	// Need to check for canceled before closing progress:
	if (progress.wasCanceled) {
		progress.close();
		return;
	}
	progress.close();

	// Check that file actually contains data
	if (numCompounds === 0) {
		MessageBox.critical(qsTr("Could not find any molecules to import."));
		return;
	}

	// Show resume dialog
	resumable = new Resumable(resumeSettingsKey, 1, numCompounds, qsTr("Import Molecules"));
	if (!resumable.init()) {
		return;
	}

	// Process file
	startTime = new Date();
	inFile = new File(inFileName);
	if (!inFile.open(File.ReadOnly)) {
		MessageBox.critical(qsTr("Could not open file '{0}'.").format(inFileName));
		return;
	}
	inStream = new TextStream(inFile);

	tempFileNamePrefix = Dir.temp() + "/" + Str.fileNameFromPath(inFileName) + "-";
	buffer = [];
	endConnectionTableWritten = false;
	ok = true;
	molCount = 0;
	successCount = 0;

	progress = new ProgressDialog();
	progress.setRange(0, resumable.maxValue - resumable.minValue + 1);
	progress.value = 0;
	progress.labelText = qsTr("Processing...");
	progress.show();

	dbDef = DBPlugin.client.definition(DBPlugin.client.database);

	while (!inStream.atEnd() && ok && !progress.wasCanceled && molCount <= resumable.maxValue) {
		line = inStream.readLine();
		//print("'" + line + "'");
		if (line === "$$$$") {
			molCount++;
			if (molCount >= resumable.minValue && molCount <= resumable.maxValue) {
				// EGW1539: Add "M  END" to connection table
				if (!endConnectionTableWritten) {
					buffer.push("M  END");
				}
				// Write temporary file
				molCountString = molCount.toString();
				while (molCountString.length < 6) {
					molCountString = "0" + molCountString;
				}
				tempFileName = tempFileNamePrefix + molCountString + ".sdf";
				//print("Writing file " + outFileName);
				tempFile = new File(tempFileName);
				tempFile.open(File.WriteOnly);
				outStream = new TextStream(tempFile);
				for (l = 0; l < buffer.length; l++) {
					outStream.writeln(buffer[l]);
				}
				outStream.writeln("$$$$");
				tempFile.close();

				// Open and save to db
				dw = Application.mainWindow.newDocument();
				if (serialization.open(tempFileName, "sdf")) {
					//print(" Opened ok: " + dw.itemCount() + " items.");
					dbItems = dbMakeDBItems(DBPlugin.client, dbDef, true, false);
					try {
						if (dbItems.length === 0) {
							throw qsTr("Nothing could be read from section.");
						}
						cid = DBPlugin.addRecord(DBPlugin.client, dbItems);
						if (cid > 0) {
							successCount++;
							resumable.setValue(successCount + 1);
							progress.labelText = qsTr("Molecule {0}: Saved {1} item(s) to record {2}.").format(molCount, dbItems.length, cid);
							Application.mainWindow.setStatusMessage(qsTr("Molecule {0}: Saved {1} item(s) to record {2}.").format(molCount, dbItems.length, cid));
							tempFile.remove();
						} else {
							throw qsTr("Error while saving {0} to database.").format(tempFileName);
						}
					} catch(err) {
						Application.mainWindow.endBusy();
						MessageBox.critical(err + "\n\n" + tempFileName);
						ok = (MessageBox.question(qsTr("Continue with Import?"), MessageBox.Yes, MessageBox.No) === MessageBox.Yes);
						Application.mainWindow.setBusy(qsTr("Working..."));
					}
				}
				dw.close();
				progress.value++;
				Application.mainWindow.processEvents();
			}
			endConnectionTableWritten = false;
			buffer = [];
		} else if (line === "M END" || line === "M  END") {
			buffer.push("M  END");
			endConnectionTableWritten = true;
		} else {
			if (line.substr(0, 1) === ">" && !endConnectionTableWritten) {
				// EGW1539: Add "M END" to connection table
				buffer.push("M  END");
				endConnectionTableWritten = true;
			}
			buffer.push(line);
		}
	}
	inFile.close();
	if (ok && !progress.wasCanceled) {
		resumable.setValue(0);
	}
	progress.close();
	Application.mainWindow.endBusy();
	serialization.fileHistory = oldFileHistory;
	MessageBox.information(qsTr("Imported <b>{0}</b> molecules into database <b>{1}</b>.").format(successCount, dbname));
	print("All done in " + ((new Date() - startTime) / 1000) + " seconds.");
}

if (this.MnUi && MnUi.scripts_db) {
	MnUi.scripts_db.scripts_db_ImportSDFile = dbImportSDFile;
}
